﻿using C1.C1Pdf;
using C1.Win.C1Document;
using C1.Win.C1Document.Util;
using C1.Win.FlexReport;
using System.Collections.Generic;
using System.Drawing;
using System.Globalization;

namespace C1FlexReportMultiDocument
{
    public static class Extension
    {
        private static C1PdfDocument pdfDocument = null;
        private static C1PdfGraphics _pdfGraphics = null;
        private static List<C1FlexReport> c1FlexReports = new List<C1FlexReport>();
        private static int page = 1;
        private static int totalPages = 0;
        public static void AddC1FlexReport(this C1PdfDocument _pdfDoc, C1FlexReport c1FlexReport)
        {
            pdfDocument = _pdfDoc;
            //Create C1PdfGraphics
            _pdfGraphics = new C1PdfGraphics(_pdfDoc);
            C1FlexReport _flex = new C1FlexReport();
            //Copy report definition
            _flex.ReportDefinition = c1FlexReport.ReportDefinition;
            //Add report to list
            c1FlexReports.Add(_flex);
        }
        public static void RenderPdf(this C1PdfDocument _pdfDoc, bool IsPageSequential = false)
        {
            //Render report(s) and calculate total page count for all report(s)
            foreach (var c1FlexReport in c1FlexReports)
            {
                c1FlexReport.Render();
                totalPages += c1FlexReport.PageCount;
            }
            //For sequential page number
            if (IsPageSequential)
            {
                foreach (var c1FlexReport in c1FlexReports)
                {
                    c1FlexReport.StartPage += C1FlexReport_StartPage;
                    c1FlexReport.Render();
                }
            }

            foreach (var c1FlexReport in c1FlexReports)
            {
                C1Document _doc = c1FlexReport.Document;
                for (int i = 0; i < c1FlexReport.PageCount; i++)
                {
                    if (_pdfDoc.Pages.Count > 1 || i > 0 || c1FlexReports.IndexOf(c1FlexReport)>0)
                    {
                        //New page added in C1PdfDocument
                        _pdfDoc.NewPage();
                    }

                    //Get C1Page from C1FlexReprot using index
                    C1Page page = c1FlexReport.GetPage(i);
                    _pdfDoc.PageSize = new SizeF((float)(Unit.Convert(page.FullWidth, _doc.MeasurementUnits, UnitTypeEnum.Point)), (float)(Unit.Convert(page.FullHeight, _doc.MeasurementUnits, UnitTypeEnum.Point)));
                    // calculate scale factor
                    double sx = _pdfDoc.PageSize.Width / Unit.Convert(page.FullWidth, _doc.MeasurementUnits, UnitTypeEnum.Dip);
                    double sy = _pdfDoc.PageSize.Height / Unit.Convert(page.FullHeight, _doc.MeasurementUnits, UnitTypeEnum.Dip);

                    // translate/scale
                    _pdfDoc.Write("{0} 0 0 {1} {2} {3} cm\r\n",
                        DoubleToString(sx), DoubleToString(sy),
                        DoubleToString(0), DoubleToString(-0 + _pdfDoc.PageRectangle.Height * (1 - sy)));

                    // draw page using pdf graphics
                    page.Draw(_pdfGraphics, new Point(0, 0), null, null, DrawAttrs.Default);
                }
                AddOutlines(_doc.Outlines);
            }
            c1FlexReports = new List<C1FlexReport>();
        }

        //Convert Double to String
        private static string DoubleToString(double _double)
        {
            return _double.ToString("0.###", CultureInfo.InvariantCulture);
        }

        private static void AddOutlines(OutlineNodeCollection onc)
        {
            foreach (var outlineNode in onc)
            {
                if (outlineNode.LinkTarget is C1LinkTarget)
                {
                    var target = ((C1LinkTarget)outlineNode.LinkTarget).Bookmark;
                    pdfDocument.AddBookmark(outlineNode.Caption, outlineNode.Level, target);
                    if (outlineNode.HasChildren)
                        AddOutlines(outlineNode.Children);
                }
            }
        }

        private static void C1FlexReport_StartPage(object sender, ReportEventArgs e)
        {
            //Update the text of page footer for printing the page number and page count in sequence
            ((sender as C1FlexReport).Fields["ftrRight"] as TextField).Text = "Page " + page + " of " + totalPages;
            page++;
        }
    }
}